#ifndef INTEGRAL_IMAGE_H
#define INTEGRAL_IMAGE_H

#include "image.h"

/**
 *This class calculate the integral image (II) of an image (I).
 *The value at position (i,j) of II is the sum of all pixel values  
 *from I that belongs to the square defined by the corners (0,0) (i,j).
 *((i,j) is included in the sum)
 *Subsequently, this structure allows a quick calculation of 
 *the sum of the value of the pixels of a square defined 
 *by corners (a,b) and (c,d)
 *
 *
 * Time-stamp: <2005-08-03 19:04:47 engmb>
 * Copyright (C) 2005 by Manuel Boissenin
 *
 * @author Manuel Boissenin
 * @date 28/07/2005
 * ***********************************************************************/

namespace mimas{

class integral_image{
 public:
 typedef int local_long_integer_type;
 typedef int local_integer_type;


  inline local_long_integer_type getPixel(const int i, const int j) const
    {
      return integralImage.getPixel(i,j); 
    }

  inline int getWidth(void) const
    {
      return integralImage.getWidth();
    }

  inline int getHeight(void) const
    {
      return integralImage.getHeight();
    }

  const int *rawData() const
    {
      return integralImage.rawData();
    }
  
  integral_image(image<unsigned char> &image);

  /*
  ~integral_image()
    {
      std::cerr<<"integral_image destructor"<<std::endl;
    }
  */
 private:
 
  image<local_long_integer_type> integralImage;
  image<local_integer_type> intermediateIntegralImage;
};

}

#endif
