#include <fstream>
#include "image_fileinput.h"
#include "linalg.h"
#include "mesa_onscreen.h"
#include "pol_model.h"
#include "pol_scene.h"
#include "x11_window.h"

using namespace boost;
using namespace mimas;
using namespace std;

class tux_window: public x11_window
{
 public:
  tux_window( x11_display *display, const image< int > &_image,
              pol_match_ptr_list _matches, line_ptr_list _ml ):
    x11_window( display, _image.getWidth(), _image.getHeight() ),
    im( _image ), matches( _matches ), ml( _ml ) {}
 protected:
  virtual void paint_event(void)
    throw (mimasexception) {

    mesa_onscreen mesa( this );
    mesa.ortho2D();

    glDisable(GL_DITHER);
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glRasterPos2i( 0, 0 );
    glPixelZoom( 1, -1 );
    glDrawPixels( width, height,
                  GL_BGRA, GL_UNSIGNED_BYTE,
		  image< rgba< unsigned char > >( im ).rawData() );
    glEnable(GL_DITHER);

    mesa.setColor( rgba< unsigned char >( 255, 0, 0 ) );

    glBegin( GL_LINES );

    for( pol_match_ptr_list::const_iterator it=matches.begin();
         it!=matches.end(); it++ ) {

      pol_match_ptr pm=(*it);
	    
      pol_match::Vector d( pm->getPosition() );
      angle a = (double)pm->getOrientation();
      double s = pm->getScale();

      //try and draw them properly now
      for( line_ptr_list::const_iterator lineit=ml.begin();
           lineit!=ml.end(); lineit++ ) {

        line_ptr l =(*lineit);
        line::Vector start(2), end(2);

        // For some reason 'rotate( l->getStart() * s, a )' won't compile in
        // release-mode.
        start = l->getStart() * s;
        end   = l->getEnd()   * s;

        start = rotate( start, -a ) + d;
        end   = rotate( end  , -a ) + d;
		  
        glVertex2i( (int)start(0), (int)start(1) );
        glVertex2i( (int)end(0)  , (int)end(1)   );
        
      }
    };

    glEnd();
    
    mesa.finish();
  };
  image< int > im;
  pol_match_ptr_list matches;
  line_ptr_list ml;
};

int main( int argc, char *argv[] )
{
  int retVal = 0;
  try {

    MMERROR( argc == 2, mimasexception, ,
             "Usage: " << argv[0] << " imagefile" << endl
             << "       searches for penguin_model in imagefile" );

    // Open X11 display.
    x11_display display;

    cout << "Creating all new model..." << endl;

    pol_model p;
    
    //switch on scale invariance AT THE MODEL...
    p.setScaleInvariance( true );
    p.createFromImageFile( "../images/stills/penguin_model.ppm" );

    line_ptr_list ml( p.getModelLines() );

    cout << "...Finished." << endl;

    pol_scene s;
  
    ifstream imfile( argv[1], ios::binary );
    image< int > im;
    imfile >> im;

    s.createFromImage(im);
    cout << "Locating model in scene..." << endl;
    pol_match_ptr_list matches( s.getMatches(p) );
    cout << "Finished searching.. did i find it?" << endl;

    // Create window.
    tux_window window( &display, im, matches, ml );

    // Loop for events.
    display.event_loop();

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
