/**********************************************************************
*Example code that shows how to use the hypothesis filter.
*
*This code is used to track a pipette tip. The tracking is done too 
*naively to give a correct result (we are tracking a translucent object)
*so the position indicated in result images is not the correct one.
*However this code should be sufficient for the 2D tracking (translation only)
*of common objects.
*
*More complex implementations are given to track succesfully the pipette.
*
*
* @author: Manuel Boissenin
* @date: 20/05/2004
*
* Time-stamp: <2005-05-20 13:18:41 engmb>
* Copyright (C) 2004 by Manuel Boissenin
 ***********************************************************************/

#include <string>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <GL/glut.h>
#include <image.h>
#include <image_fileinput.h>
#include <image_fileoutput.h>
#include <image_mesaoutput.h>

#include "condensation.h"
#include "hypothesis2D.h"
#include "correlation_observer2D.h"
#include "analyse_result_mean.h"
#include "image_loader.h"

using namespace mimas::hf;
using namespace std;

int main(int argc, char** argv)
{
  int retVal = 0;
  try {
    glutInit( &argc, argv );
    x11_display disp;    
    image_mesaoutput<unsigned char > win( &disp );

    hypothesis2D init(atoi(argv[2]),atoi(argv[3]),1.);
    hypothesis2D::sigmaSpan = atof(argv[4]);

    //parametres for the different part of the host class.
    image_loader<unsigned char>::set il_param(argv[1], 99, ".png");
    correlation_observer2D<hypothesis2D, unsigned char>::set obs_param(atoi(argv[2]),atoi(argv[3]), 40, 40);
    analyse_result_mean<hypothesis2D, unsigned char>::set analyse_param;
    
    hypothesis_filter<image_loader, correlation_observer2D, analyse_result_mean, hypothesis2D>
      hf(init, atoi(argv[5]), il_param, obs_param, analyse_param);

    int imageNumber = 0;
    
    while(true){
      try{hf.next();}
      catch (mimasexception &e){
	std::cerr<<e.what()<<std::endl;  
	return 0;
      }
      
      hypothesis2D estimatedLocation;
      hf.filter();
      estimatedLocation = hf.analyse();
      
      hf.draw(estimatedLocation);//from Observer

    
      std::cerr<< "Image size: "<< hf.image.getWidth() <<", "<< hf.image.getHeight()<<std::endl; 
      win << hf.image<< mimas::delay(100);


      //----------save on a file--------//
      
      std::ostringstream str;
      str << "results/pipetteMeanEstimation" << std::setw(3) << std::setfill( '0' ) <<  imageNumber++
	  << ".ppm";
      std::ofstream filteredOut(str.str().c_str(), std::ios::binary); 
      filteredOut << hf.image;
      //---------------------------------//
    }
  } catch ( exception &e )
    {
      cerr << "An exception occurred: " <<  e.what() << endl;
      retVal = 1;
    };
  return retVal;
}
