#ifndef __GNUPLOT_TCC
#define __GNUPLOT_TCC

template<typename T>
void gnuplot::plot_histogram(const std::vector<T> &histogram, std::string title)
{
  //get the "\n" changed to "\\n" and add " at the beginning and end of the string 
  std::string the_title;
  std::istringstream iss( title.c_str() );
  std::string line;
  while ( std::getline( iss, line ) )
    {
      the_title += line + "\\n";
    }
  program<<"set title \"";  
  program<<the_title<<"\""<<std::endl;

  program<<"unset key"<<std::endl;
  program<<"set style data hist" <<std::endl;
  program << "plot '-' " << std::endl;
  for(int index = 0; index < histogram.size(); index++)
    program<<index + .5 << " "<<histogram[index] << std::endl;

  program << "e" << std::endl;
}

template<typename T>
void gnuplot::plot(const std::vector<T> &vector, std::string title)
{
  //get the "\n" changed to "\\n" and add " at the beginning and end of the string 
  std::string the_title;
  std::istringstream iss( title.c_str() );
  std::string line;
  while ( std::getline( iss, line ) )
    {
      the_title += line + "\\n";
    }
  program<<"set title \"";  
  program<<the_title<<"\""<<std::endl;

  program<<"unset key"<<std::endl;
  program << "plot '-' with lines " << std::endl;
  for(int index = 0; index < vector.size(); index++)
    program<<index << " "<<vector[index] << std::endl;

  program << "e" << std::endl;
}


template< typename T >
void gnuplot::plot( const boost::numeric::ublas::vector< T > &vec )
{
  program << "plot '-' w lines" << std::endl;
  for (int i=0; i<(signed)vec.size(); i++ )
    program << std::fixed << std::setprecision( DBL_DIG ) << vec( i ) 
            << std::endl;
  program << "e" << std::endl;
}

template< typename T >
void gnuplot::plot( const boost::numeric::ublas::matrix< T > &mat )
{
  program << "plot '-' w lines";
  for (int j=1; j<(signed)mat.size2(); j++)
    program << ", '-' w lines";
  program << std::endl;
  
  for (int i=0; i<(signed)mat.size2(); i++) {
    for (int j=0; j<(signed)mat.size1(); j++)
      program << std::fixed << std::setprecision( DBL_DIG ) << mat( j, i ) 
              << std::endl;
    program << "e" << std::endl;
  }
}

template< typename T >
void gnuplot::plot( const boost::numeric::ublas::matrix< T > &mat,
                       int xcol, int ycol,
                       const std::string &style, int type, int size )
  throw (mimasexception)
{
   MMERROR( mat.size1() > 0 && mat.size2() > 0 &&
            (signed)mat.size1() >= xcol && (signed)mat.size1() >= ycol,
            mimasexception, , "gnuplot::plot - input matrix error" );

   assert( style == "lines" || style == "points" );
   program << "plot '-' w " << style << " lt " << type << " lw " << size
           << std::endl;
	
   for ( int j=0; j<(signed)mat.size1(); j++ ) {
     program << std::fixed << std::setprecision( DBL_DIG ) << mat( j, xcol ) << " "
             << std::fixed << std::setprecision( DBL_DIG ) << mat( j, ycol ) << std::endl;
   };     

   // tell gnuplot that it has reached the end of the dataset
   program << "e" << std::endl;
}

template< typename T >
void gnuplot::plotxy( const boost::numeric::ublas::matrix< T > &mat,
                      int xcol, int ycol, ... ) throw (mimasexception)
{
  MMERROR( mat.size1() > 0 && mat.size2() > 0,
           mimasexception, , "gnuplot::plotxy - input matrix error" );
  
  MMERROR( xcol>=0 && xcol<(signed)mat.size2() &&
           ycol>=0 && ycol<(signed)mat.size2(),
           mimasexception, , "gnuplot::plotxy - illegal column values" );
  
  va_list args;
  
  va_start(args,ycol);
  
  // precount parameters
  int paramcount=-1;
  int val=-1;
  while(val!=0) {
    paramcount++;
    val=va_arg(args,int);
  }
  
  va_end(args);
  

  // load the parameters in ycols
  paramcount++; // to take into account the first ycol parameter
  int ycols[paramcount];
  ycols[0]=ycol;
  
  va_start(args,ycol);
  
  // start from 1 since first parameter already loaded
  for (int i=1; i<paramcount; i++) {
    val=va_arg(args,int);
    MMERROR( val>=0 && val<(signed)mat.size2(), mimasexception, ,
             "gnuplot::plotxy - illegal column values for parameter "
             << i+3 << " or parameters not NULL terminated" );
    ycols[i]=val;
  } 
  
  va_end(args);

  // do the plotting here;	
  char buf[32767];
  
  if ( lineTitles.empty() )
    sprintf(buf,"plot '-' title \"column %d\" w lines", ycols[0]);
  else
    sprintf(buf,"plot '-' title \"%s\" w lines", lineTitles[0].c_str());
  program << buf;
  for (int j=1; j<paramcount; j++)
    {
      if ( j >= (signed)lineTitles.size() )
        sprintf(buf,", '-' title \"column %d\" w lines", ycols[j]);
      else
        sprintf(buf,", '-' title \"%s\" w lines", lineTitles[j].c_str());
      program << buf;
    }
  program << std::endl;
  
  for (int i=0; i<paramcount; i++)
    {
      for (int j=0; j<(signed)mat.size1(); j++)
        {
          sprintf(buf,"%2.20f %2.20f",(double)mat(j,xcol),
                  (double)mat(j,ycols[i]));
          program << buf << std::endl;
        }

      program << "e" << std::endl;
    }
}

template< typename T >
void gnuplot::plotImageProfile( const image< T > &im )
{
  char conv[32767];
  
  if (!im.initialised()) return;
  
  program << "splot '-' mat w lines" << std::endl;
	
  for (int j=0; j<im.getHeight(); j++)
    {
      for (int i=0; i<im.getWidth(); i++)
        {
          sprintf(conv,"%g",(const double)im.getPixel(i,j));
          program << " " << conv << std::endl;
        }
      program << std::endl;
      
    }
  program << "e" << std::endl;
}

template< typename T >
void gnuplot::plotImageProfile2(const image< T > &im, std::string title )
{
  if (!im.initialised()) return;
  
  program << "set title \""<< title<<"\"" <<std::endl;
  program << "unset key"<<std::endl;
  program << "splot '-' " << std::endl;
	
  for (int j=0; j<im.getHeight(); j++)
    for (int i=0; i<im.getWidth(); i++)
        {
	  int n;
          if ((n = im.getPixel(i,j)) != 0){
	    program <<i<<" " <<j<<" " << n << std::endl;
	  }	  
	}

  program << "e" << std::endl;
}

#endif
