#ifndef IMAGE_MPLAYEROUTPUT_H
#define IMAGE_MPLAYEROUTPUT_H

#include "mimasconfig.h"
#ifdef HAVE_MKSTEMP
#include "image_output.h"
#include "rgba.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageOutput
    @{ */
/** Display tool that sits over Mplayer.
    @author Bala Amavasai (bala@amavasai.org)
    @date Fri May 14 19:06:10 BST 2004
    $Header: /cvs/mimas2/include/image_mplayeroutput.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $ */
template< typename T >
class image_mplayeroutput: public image_output< T >
{
 public:
  /** Constructor.
      @param _width Width of display.
      @param _height Height of display. */
  image_mplayeroutput( int _width, int _height )
    throw (mimasexception);
  /// Destructor.
  virtual ~image_mplayeroutput(void);
  ///
  virtual void write( const const_image_ref< T > &image );
 protected:
  /// Width of display.
  int width;
  /// Height of display.
  int height;
  ///
  std::string pipename;
  /// The pipe for images.
  int imgpipe;
  /// The pipe for opening MPlayer.
  FILE *mplayer;
};

template<>
void image_mplayeroutput< rgba< unsigned char > >::write
  ( const const_image_ref< rgba< unsigned char > > &image );
template<>
void image_mplayeroutput< unsigned char >::write
  ( const const_image_ref< unsigned char > &image );

/// Stream-operator for displaying image with mplayer.
template< typename T, typename U >
inline image_mplayeroutput< T > &operator<<
  ( image_mplayeroutput< T > &output,
    const const_image_ref< U > &im )
{
  if ( typeid( T ) == typeid( U ) )
    output.write( (const_image_ref< T > &)im );
  else {
    output.write( image< T >( im ) );
  };
  return output;
}

///@}

///@}

};

#include "image_mplayeroutput.tcc"
#endif

#endif
