namespace mimas {

template< typename T >
image_xineinput< T >::image_xineinput( const std::string &_mrl,
                                       const char *vo_driver,
                                       const char *ao_driver )
  throw (mimasexception):
  mrl(_mrl), xine(NULL), vo_port(NULL), ao_port(NULL), stream(NULL),
  width(-1), height(-1), format(-1)
{
  MMERROR( vo_driver != NULL && strcmp( vo_driver, "null" ) != 0,
           mimasexception, ,
           "xine video output driver was \"null\". Use \"auto\" instead." )

  bool streamOpen = false;

  // Create xine-object.
  xine = xine_new();

  try {

    // Load configuration file.
    std::ostringstream configFile;
    configFile << xine_get_homedir() << "/.xine/config";
    xine_config_load( xine, configFile.str().c_str() );

    // Call post-init.
    xine_init(xine);
  
    // Open xine video output port.
    vo_port = xine_open_video_driver(xine, vo_driver, XINE_VISUAL_TYPE_NONE,
                                     NULL );
    if ( ao_driver != NULL )
      ao_port = xine_open_audio_driver(xine , ao_driver, NULL);

    // Open a xine stream connected to the output port(s).
    stream = xine_stream_new(xine, ao_port, vo_port);
    MMERROR( stream != NULL, mimasexception, ,
	     "Error creating xine stream." );

    MMERROR( xine_open( stream, mrl.c_str() ), mimasexception, ,
	     "Error opening xine stream (xine_get_error() -> "
	     << xine_get_error( stream ) << ")." );
    streamOpen = true;

    MMERROR( xine_play( stream, 0, 0 ), mimasexception, ,
	     "Error playing stream (xine_get_error() -> "
	     << xine_get_error( stream ) << ")." );

  } catch ( mimasexception &e ) {

    if ( streamOpen ) xine_close( stream );
    if ( stream ) xine_dispose( stream );
    if ( ao_port ) xine_close_audio_driver(xine, ao_port);
    if ( vo_port ) xine_close_video_driver( xine, vo_port );
    if ( xine ) xine_exit(xine);
    throw e;

  };
}

template< typename T >
image_xineinput< T >::~image_xineinput(void)
{
  if ( stream ) {
    xine_close( stream );
    xine_dispose( stream );
  };
  if ( ao_port ) xine_close_audio_driver(xine, ao_port);
  if ( vo_port ) xine_close_video_driver( xine, vo_port );
  if ( xine ) xine_exit(xine);
}

};

