#ifndef MESA_ONSCREEN_H
#define MESA_ONSCREEN_H

#include "mimasconfig.h"

#ifdef HAVE_OPENGL

#include "mesa_base.h"
#include "x11_window.h"

namespace mimas {

/** Mesa on-screen rendering.
    Support for on-screen rendering using Mesa.

    No stencil- and accumulation-buffer are allocated.

    Do not forget to initialise <EM>glut</EM> at the beginning of your
    program:
    \verbatim
    glutInit( &argc, argv );
    \endverbatim

    If you experience problems, you may want to try:
    \verbatim
    export LIBGL_ALWAYS_INDIRECT=1
    \endverbatim
    This disables hardware rendering.

    If it works:
    - the problem is likely with the display driver
    - it will be very slow

    If it doesn't:
    - there is a different bug, we need to fix

    @see x11_window
    @author Jan Wedekind (jan at wedesoft.de)
    @date Wed Mar 30 19:35:05 UTC 2005 */
class mesa_onscreen: public mesa_base
{
 public:
  /** Constructor.
      All parameters must be destructed by the calling program.
      @param _window Window to open GLX context for. */
  mesa_onscreen( x11_window *_window ) throw (mimasexception);
  /** Destructor.
      The destructor will destruct the OpenGL-context and call
      \c mesa_base::finish() . */
  virtual ~mesa_onscreen(void);
  /** Finish on-screen rendering.
      This method calls \c glFinish() and \c mesa_base::finish() . */
  virtual void finish(void);
  /** Switch to two-dimensional drawing mode.
      @see mesa_base::ortho2D */
  virtual void ortho2D( int x1, int y1, int x2, int y2 ) const;
  /** Switch to two-dimensional drawing mode.
      @see mesa_base::ortho2D */
  virtual void ortho2D(void) const;
 protected:
  ///
  x11_window *window;
  /// GLX context for on-screen rendering.
  GLXContext context;
  /// State of object.
  bool finishOnScreen;
};

};

#endif

#endif

