#ifndef __REGISTRATION_H
#define __REGISTRATION_H

#include "mimasconfig.h"

#ifdef HAVE_LIBFFTW3

#include "object.h"
#include "image.h"
#include "canny.h"

namespace mimas{

/** Class denoting shift, rotation and scale.
    @see registration_parameters */
class reg_param
{
 public:
  int shift_x;
  int shift_y;
  double angle;
  double scale;
  reg_param(int x = 0, int y = 0, double a = 0, double sc = 0)
    :shift_x(x),shift_y(y),angle(a),scale(sc){}
};

/** The executable in 'registration' takes the two test images
    in parameters and the size of the square area two be used for
    registration.

    References:
    <ul>
    <li>Xie H., Hicks N., Keller G.R., Huang H. and Kreinovich V., <A HREF="http://www.nmt.edu/~hjxie/Auto.pdf">Automatic Image Registration Based on A FFT Algorithm and IDL/ENVI</a></li>
    <li>Reddy B.S., Chatterji B.N., <li><a href="http://www.tsi.enst.fr/~soler/prj/m_apical/Reddy_FFTregistration.pdf">An FFT-Based Technique for Translation, Rotation, and Scale-Invariant Image Registration</a></li>
    </ul>

    @see reg_param
    @author Denis Tessier
    @date Sep 2004 */
class registration_parameters : public object
{
private :
  double angle;
  double scale;
  int shiftx;
  int shifty;
 
 public:
  void setAngle(double a)     {angle = a;}
  void setScale(double s)     {scale = s;}
  void setShift(int x, int y) {shiftx = x; shifty = y;}
 
  double getAngle(){return angle;}
  double getScale(){return scale;}
  int getShiftx(){return shiftx;}
  int getShifty(){return shifty;}

  reg_param getRegParam(){return reg_param(shiftx, shifty, angle, scale);}
  
  void registering_images(image<int>& im1, image<int>& im2);
  void registering_images_shift_only(image<int>& im1, image<int>& im2);
 
  registration_parameters(void){angle = 0.0; scale = 1.0; shiftx = 0; shifty = 0;}
  virtual ~registration_parameters(void) {}
};

///
image<int> registration(image<int>& inimage, image<int>& inimage2, reg_param *result_p = NULL, bool with_scale_and_rotation = true);

/** Registration using edge-images.
    When the images have little texture this method may succed better to register both images.
    It adds edge features to both images thus increasing texture and likelyhood of success
    of the registration.
    @date 1 Sep 2005
    @author Manuel Boissenin */
image<int> registration_with_canny(image<int>& inimage1, image<int>& inimage2,int lwThresh = 40, int highThresh = 110, float stdDev = 1.2
				      ,reg_param *result_p = NULL, bool with_scale_and_rotation = true);

}

#endif

#endif

